/*
 * 
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 2007-2008 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 *
 */
package com.sun.grizzly.samples.migration.command;

import com.sun.grizzly.samples.migration.QuoteManager;
import com.sun.grizzly.samples.migration.connection.handler.ClientConnectionHandler;
import com.sun.grizzly.samples.migration.connection.handler.ThirdPartyConnectionHandler;
import com.sun.grizzly.samples.migration.response.handler.FeedResponseHandler;
import com.sun.grizzly.samples.migration.response.handler.IResponseHandler;

/**
 * The feed Command must be in the format
 * 
 * feed|symbol[eoq]
 * 
 * The server will return the latest quote from
 * the cache, if there is a feed subscribe for this
 * symbol.  If not, the server will ask the quote at
 * the 3th party, and when the response is received,
 * it will be send to be client.
 * 
 * The connection with the client won't be close like
 * the quote command.
 * 
 * @author Sebastien Dionne
 *
 */
public class FeedCommand extends QuoteCommand {
	
	public FeedCommand(QuoteManager manager) {
		super(manager);
	}

	/**
	 * process the query
	 * @param query query to send to 3th party
	 * @param clientConnectionHandler client that call the command
	 * @return CommandRequest the request 
	 */
	public ICommandRequest process(String query, ClientConnectionHandler clientConnectionHandler){
		
		// extract the quote from the query
		int index = query.indexOf(QuoteManager.ARG_SEP);
		if(index<=0){
			//error
			return null;
		}
		
		f_quoteSubscription = query.substring(index+QuoteManager.ARG_SEP.length());
		
		f_responseHandler = new FeedResponseHandler();
		f_responseHandler.setClientConnectionHandler(clientConnectionHandler);
		f_responseHandler.setCommand(this);
		
		
		// add the quoteSubscription to the Cache for this client
		f_manager.addQuoteSubcriptionForClientConnectionHandler(f_quoteSubscription, clientConnectionHandler);
		
		// look if the quoteSubscription is not in the cache
		IResponseHandler handler = f_manager.getResponseHandlerFromQuoteSubcription(f_quoteSubscription);
		if(handler!=null){
			System.out.println("QUOTE FEED FOUND IN CACHE = " + f_quoteSubscription);
			
			handler.sendLastUpdateToClient(clientConnectionHandler);
			return handler.getCommandRequest();
		} 
		
		ThirdPartyConnectionHandler connection = f_manager.getThirdPartyConnectionHandler();
		
		if (connection == null) {
			return null;
		}

		// Create a new request to the 3th party
		try {
			f_manager.addQuoteSubcription(f_quoteSubscription, f_responseHandler);
			return connection.sendFeedRequest(f_quoteSubscription, f_responseHandler);
		} catch (Exception e) {
			// here we could send a message to the client.. but it's a demo
			System.out.println(e);
		}
		
		return null;
	}

}
